function[standardFFT,timeFFT,freqFFT,FFTSize,FFTInput] = reassignmentAnalysis(frame,zeroPad);
%[standardFFT,timeFFT,freqFFT] = reassignmentAnalysis(frame, zeroPad);
%Takes a frame of time domain data
%and performs Reassignment Analysis
%frame: vector containing time domain data
%zeroPad: zero-padding factor (1x no zeroPadding)
%If the frame length is not a power of 2 then it is 
%zero-padded to the next power of 2
%Three complex vectors are returned:
%standardFFT: FFT of Hann windowed frame
%timeFFT: FFT of time-weighted Hann windowed frame
%freqFFT: FFT of frequency-weighted Hann windowed frame

%calculate standard FFT
N = length(frame);
arg = linspace(-1,1,N);
hann = 0.5 + (0.5*(cos(pi.*arg)));
windowed = hann .* frame;
%now generate zero-padded window
FFTReady = zeroPadPhase(windowed,zeroPad);
temp = fft(FFTReady);
standardFFT = temp(1:(round(length(temp)/2))+1);
clear temp;
FFTInput = FFTReady;

%calculate time ramped FFT
range = (N-1)/2;
tHann = hann .* [-range:range];
windowed = tHann .* frame;
%now generate zero-padded window
[FFTReady,FFTSize] = zeroPadPhase(windowed,zeroPad);
temp = fft(FFTReady);
timeFFT = temp(1:(round(length(temp)/2))+1);
clear temp;

%calculate frequency ramped FFT 
dHann = 0.5*-sin(pi.*arg)*(2*pi/N); % derivative of Hann window
windowed = dHann .* frame;
%now generate zero-padded window
FFTReady = zeroPadPhase(windowed,zeroPad);
temp = fft(FFTReady);
freqFFT= temp(1:(round(length(temp)/2))+1);








