function[frame] = SineGenerate(f,a,df,da,N,startPhase,Fs);
%[frame] = SineGenerate(f,a,df,da,N,startPhase,Fs);
% Returns a single frame of a sinusoid with characteristics:
% f: mid-point frequency (Hz)
% a: mid-point amplitude
% df: frequency change within frame (Hz)
% da: amplitude change within frame (dB)
% N is window size
% startPhase is phase of first sample
% Fs is sample rate


%Fs is sampling frequency

if (nargin<7)
    Fs = 44100;
end
doublePi = 2 * pi;
%is amp ramp linear (exp ~= 1.0) or exponential (exp == 1.0)
exp = 1.0;

if (df == 0)
   %generate phases for stationary sinusoid
    if (f ~= 0)
        phases = startPhase: ((f/Fs) * doublePi): ((N-1) * ((f/Fs) * doublePi) + startPhase);
    else
        % generate vector of identical phase values
        phases = startPhase.*ones(1,N);
    end
else
   %generate phases for sinusoid with frequency ramp
   phases  = zeros(1,N);
   phases(1,1) = startPhase;
   for n = 2:N
      phases(1,n) = phases(1,n-1) + ((((f - 0.5*df) +((n-2)/(N-2)* df))/Fs) * doublePi);
   end
end
%generate sinusoid
sinusoid = sin(phases);

if(da ~= 0.0)
   if(exp ~= 1.0)
   	%apply linear amplitude modulation to sinusoid
	   %do in two halves
	   da = 0.5 * da;
 		%convert from dB
  	 	ampChange = 10^(da/20);
  		ampRamp = linspace((1/ampChange),ampChange,N);
  	 	%scale overall amplitude so that a is still the mean amplitude
  		m = mean(ampRamp);
   	ampRamp = (a/m) * ampRamp;
   	dBdiff = 20*log10(ampRamp(N)/ampRamp(1));
      sinusoid = sinusoid .* ampRamp; 
   else
      %apply exponential amplitude modulation to sinusoid
      %do in two halves
      da = 0.5 * da;
      %convert from dB
      ampChange = 10^(da/20);
      ampRamp = logspace(log10(1/ampChange),log10(ampChange), N);
      % scale overall amplitude so that a is still the mean amplitude
  	  % calculate mean
      m = (10/log(10))*(ampChange - (1/ampChange))/da;
      mn = mean(ampRamp);
   	  ampRamp = (a/m) * ampRamp;
   	  dBdiff = 20*log10(ampRamp(N)/ampRamp(1));
      sinusoid = sinusoid .* ampRamp; 
   end
else
   sinusoid = sinusoid * a;
end
frame = sinusoid;
%plot(sinusoid);
