function[dfEst, daEst, chiSquared] = Wells(standardFFT,freqFFT,timeFFT, FFTSize,fVector,peakIndex,zeroPad,range);
%[dfEst, daEst] = Wells(standardFFT,freqFFT,peakIndex,zeroPad);
%Performs Wells (RDA) frequency and amplitude modulation analysis
%by fitting a curve to the frequency reassignment data
%around a zero-padded peak
%range: the number of zero-padded bins either side of the peak
%taken in the analysis

doublePi = 2 * pi;
upperLimit = length(fVector);
Fs = 44100;
binWidth = Fs/FFTSize;

%obtain plot coordinates
lowerRange = peakIndex - range;
upperRange = peakIndex + range;

if (lowerRange < 1) 
   lowerRange = 1;
end
if (upperRange > upperLimit) 
   upperRange = upperLimit;
end
x = zeros(1,(2*range)+1);   
y = zeros(1,(2*range)+1);   
for (a = lowerRange:upperRange)
   x(a-lowerRange+1) = (binWidth * (a-1)) - fVector(peakIndex);
   y(a-lowerRange+1) = real(timeFFT(a)/standardFFT(a));
end
%do second-degree polynomial fitting
roots = polyfit(x,y,2);   % change this back
dfEst = roots(2);
daEst = roots(3);

%calculate error over greater range
range = range+1;
lowerRange = peakIndex - range;
upperRange = peakIndex + range;

if (lowerRange < 1) 
   lowerRange = 1;
end
if (upperRange > upperLimit) 
   upperRange = upperLimit;
end
x = zeros(1,(2*range)+1);
y = zeros(1,(2*range)+1);
for (a = lowerRange:upperRange)
   x(a-lowerRange+1) = (binWidth * (a-1)) - fVector(peakIndex);
   y(a-lowerRange+1) = real(timeFFT(a)/standardFFT(a));
end
%calculate sum of the squared residuals
residual = (y(2:2:end) - polyval(roots,x(2:2:end))).^2;  % change this back
chiSquared = sum(residual);

