function[amp] = CorrectAmp(peakAmp,freq,FFTSize,frameSize,ampFactor); 
%[amp] = CorrectAmp(peakAmp)
%This function performs amplitude correction on a DFT
%amplitude estimate (taken from the peak bin) according to 
%the shape of the main lobe of the window
%This method is described in JAES 2000 article by Marchand &
%Desainte-Catherine.
%f MUST be the corrected frequency 
%i.e. not just the centre frequency of the peak bin
% ampFactor reduces the amount of correction where the
% window has been flattened by non-stationarity

doublePi = 2 * pi;
Fs = 44100;
binWidth = Fs/FFTSize;

%calculate deviation from centre of the nearest bin
deviation = freq/binWidth;
deviation = deviation - floor(deviation);
if (deviation > 0.5)
   deviation = 1.0 - deviation;
end
deviation = (deviation * frameSize)/FFTSize;

%scale the amplitude
deviationSquared = deviation^2;
if (deviation == 0.0)
   amp = peakAmp;	%no need to correct estimate
else
   scaler = (sin(pi * deviation))/((pi * deviation) * (1 - deviationSquared));
   amp = peakAmp/(scaler^(1/ampFactor));
end
