function[output,input] = dfRemoval(input)
% takes an input signal (which should be a single analysis frame) and
% removes intra-frame amplitude change from invidual components
% NB this code is for illustrative purposes only and so contains only
% minimal error checking
% Jez Wells, University of York, 2010.

if(min(size(input))>1)
    error('function only works with single channel input');
end
N = length(input);
complexSize = round(N*0.5)+1;
input = input.*hann(N)';
input = circshift(input,[0,ceil(N*0.5)]);   % apply zero-phase window
INPUT = fft(input);
mag = abs(INPUT);
ph = unwrap(angle(INPUT));

% find component regions
% get first order difference of quantized magnitude spectra
diffMag = diff(mag(1:complexSize)); % just do one side of the spectrum
% apply mask
diffMag(find(diffMag>0)) = 1;
diffMag(find(diffMag<0)) = -1;
% find differential of mask to get stationary points
diffMag = diff(diffMag);
    
lower = [1;find(diffMag==2)]; % -2 will be maxima, +2 will be minima
upper = [find(diffMag==2)+1;complexSize]; 

% now deal with each component region in turn
for n = 1:length(lower)
    if(upper-lower>1)   % need at least three points for a second order polynomial
        % determine local indices
        [m,i] = max(mag(lower:upper));
        if(upper(n)-lower(n)>4)
            if(i+2>upper)
                local = upper-3:upper;
            elseif(i-2<lower)
                local = lower:lower+3;
            elseif(mag(i-2)>mag(i+2))
                local = i-2:i+1;
            else
                local = i-1:i+2;
            end
            non_local = [lower:local(1)-1,local(end)+1:upper];
        else
            local = lower:upper;
            non_local = [];
        end
        % determine offset of component from bin centre
        % do this via parabolic interpolation
        X = log10(mag(i-1:i+1));
        d = 0.5*(X(1)-X(3))/(X(1)-(2*X(2))+X(3));
        % phase processing
        % determine coefficients of local phase polynomial
        p = polyfit(local-i-d,ph(local),2);
        % set slope to 0 and then reconstruct
        p(1) = 0;
        ph(local) = polyval(p,local-i-d);
        % magnitude processing
        bins = abs(i+d-local);
        actual_mag = 10^(X(2)-(d*0.25*(X(1)-X(3))));
        mag(local) = abs(sin(pi*bins)./(2*pi*bins.*(1-(bins.^2))));
        % now scale according to peak (peak value of the above expression
        % for a Hann windowed stationary sinusoid is 0.5)
        mag(local) = mag(local)*actual_mag*2;
        %ph(non_local) = ph(i)*(-1.^(i-non_local));
    end
end

% recombine modfied magnitude and phase
rl = mag.*cos(ph);
im = mag.*sin(ph);
INPUT = complex(rl,im);
output = ifft(INPUT, 'symmetric');   
% undo zero-phase windowing
output = circshift(output,[0,-ceil(N*0.5)]);
input = circshift(input,[0,-ceil(N*0.5)]);
            
        
    
