function[] = lookupTableGen()
% function[] = lookupTableGen()
% function to generate lookup tables for sinusoidal descrption and
% discrimination (df,da, chiSquared etc.)

range = 1; % number of values taken either side of sinusoidal peak
dim = 100; % array dimensions

zeroPad = 8;
N = 1025; % frame size
NFFT = 2^(floor(log2(N * zeroPad))); % FFT size
NW = 2; % Thomson analysis bandwidth
freq = (NFFT/4)*(44100/NFFT);   % centre of analysis bin in centre of spectrum

contNFFT = 2^17;    % 'continuous' FFT size
contBin = (contNFFT/4)+1; % +1 because indexing starts at 1

% values of da and df at which quantities are sampled
dfX = linspace(0,260,dim);
daX = linspace(0,96,dim);

% initialise matrices for speed
dfEst = zeros(dim,dim);
daEst = zeros(dim,dim);
chiSquared = zeros(dim,dim);
second_deriv = zeros(dim,dim);
reassignment_difference = zeros(dim,dim);
%thomson_F_test = zeros(dim,dim);
thomson_V0_1 = zeros(dim,dim);
thomson_V0_2 = zeros(dim,dim);
thomson_V0_3 = zeros(dim,dim);

% generate the df, da and chiSquared lookup tables
for n = 1:length(dfX)
    tic
    for m = 1:length(daX)
        % calculate the un-weighted mean frequency required to give the
        % desired weighted mean frequency (so that peak is in centre of
        % bin)
        compensatedFreq = actualFreqMean(freq,dfX(n),daX(m));
        % check that the peak is in the correct place
        s = sineGenerate2(compensatedFreq,1,dfX(n),daX(m),N,0,44100).*hann(N)';
        S = fft(s,contNFFT);
        [dum,idx] = max(S);
        change = 5; % resset increment/decrement amount
        prevLess = 2; % initialise so that it is neither 0 nor 1 
        while(idx~=contBin) % peak is not at correct place in 'continuous' spectrum
            % increment or decrement unweighted frequency
            if(idx<contBin) 
                compensatedFreq = compensatedFreq+change;
                if(prevLess==0)
                    % change in direction so reduce range travelled
                    change = change*0.5;
                end
                prevLess = 1;
            else
                compensatedFreq = compensatedFreq-change;
                if(prevLess==1)
                    % change in direction so reduce range travelled
                    change = change*0.5;
                end
                prevLess = 0;
            end
            s = sineGenerate2(compensatedFreq,1,dfX(n),daX(m),N,0,44100).*hann(N)';
            S = fft(s,contNFFT);
            [dum,idx] = max(S);     
        end           
        % generate the df, da and chiSquared lookup tables
        %compensatedFreq = compensatedFreq + (0.5*44100/8192);    % temporary line to see what happens half way between bins
        [fEst,aEst,peakAmp, dfEst(m,n), daEst(m,n), phiExpect(m,n), chiExpect(m,n), standardFFT, timeFFT, freqFFT, fVector, frame,ind(m,n)] = PeakAnalysisWells(compensatedFreq,1,dfX(n),daX(m),0,range);
        % generate the second derivative lookup table
        [temp, Xo] =  mixedpd_jezmod(frame,NFFT,44100,hanning(N));
        [dum,i] = max(Xo);
        secondDerivExpect(m,n) = temp(i);
        % generate reassignment difference lookup table
        reassDiffExpect(m,n) = reassDiff(standardFFT, timeFFT, freqFFT); 
        % generate Thomson F-test table
        %[dum,thomson_F_test(m,n)] = thomsonMTM(frame,NFFT,NW,i);
        % generate Thomson V0 estimates
        temp = thomsonMTM_V0generate(frame,N,NFFT,NW,dfX(n),daX(m));
        thomson_V0_1(m,n) = sqrt(temp(1));
        thomson_V0_2(m,n) = sqrt(temp(2));
        thomson_V0_3(m,n) = sqrt(temp(3));
        % generate Masri tables
        [dfEstMasri(m,n), daEstMasri(m,n)] = Masri(standardFFT,ind(m,n), zeroPad);
    end
    n
    toc
end

figure
mesh(ind)

save ('tables','phiExpect','dfX','daX','dfEst','daEst','chiExpect','secondDerivExpect','reassDiffExpect','thomson_V0_1','thomson_V0_2','thomson_V0_3','dfEstMasri','daEstMasri')

function[reass_difference] = reassDiff(standardFFT, timeFFT, freqFFT)

[dum,index] = max(standardFFT);
phaseOffset = real(timeFFT(index)./standardFFT(index));
ampOffset = real(freqFFT(index)./standardFFT(index));
reass_difference = ((8192 * pi).*ampOffset) + phaseOffset;


