function[lower,upper,maxima] = regionFind(input,zeroPad);
% [lower,upper] = regionFind(input);
% function that takes a single vector and locates
% minima within it. The area between these minima is a single 'region'.
% The maxima within those regions is also returned.
% Designed for finding regions and their peaks in Fourier magnitude spectra.
% It takes account of zero-padding by avoiding classifying side-lobe dips as minima

N = length(input);

method = 'energy';  % use moving average of the energy
% method = 'local maxima';  % find local maxima and then use these to find
% global minima

if(rem(zeroPad,2))
    warning('zeroPad is expcted to be an even number');
end


if(zeroPad>1)
    if(strcmp(method,'local_maxima'))
        numRegions = N/zeroPad;
        local_max = zeros(1,ceil(numRegions));
        local_min = zeros(1,ceil(numRegions));
        % indices
        local_min_idx = zeros(1,ceil(numRegions)); 
        local_max_idx = zeros(1,ceil(numRegions)); 
        % find local peaks
        local_min_idx(1) = 1;   % first region starts at DC
        [local_max(1),local_max_idx(1)] = max(input(1:zeroPad));
        for n = 2:floor(numRegions)
            [local_max(n),local_max_idx(n)] = max(input(((n-1)*zeroPad)+1:n*zeroPad));
            [local_min(n),local_min_idx(n)] = min(input(((n-1)*zeroPad)+1:n*zeroPad));
            % index returned by min/max is only local so need to add global offset
            local_min_idx(n) = local_min_idx(n)+(n-1)*zeroPad;
            local_max_idx(n) = local_max_idx(n)+(n-1)*zeroPad;
        end
        if(n~=ceil(numRegions))
            n = n+1;
            [local_max(n),local_max_idx(n)] = max(input(((n-1)*zeroPad)+1:end));
            local_min_idx(n) = N; % last region ends at Nyquist
        end
        % now find global minima
        temp = diff(local_max);
        % apply mask
        temp(find(temp>0)) = 1;
        temp(find(temp<0)) = -1;
        % find differential of mask to get stationary points
        temp = diff(temp);
        lower = local_min_idx([1,find(temp==2)]+1); % -2 will be maxima, +2 will be minima
        maxima = local_max_idx([find(temp==-2)]+1);
        upper = local_min_idx([find(temp==2)+1,n])-1; 
        if(maxima(1)>upper(1))
         % ensure that a maximum is located in the first region
            [m,i] = max(input(lower(1):upper(1)));
            maxima = [i,maxima];
        end
        if(maxima(end)<lower(end))
            % ensure that a maximum is located in the last region
            [m,i] = max(input(lower(end):upper(end)));
            maxima(end+1) = i;
        end   
    else
        filterLen = zeroPad;    % length of moving average filter
        % create smoothed energy spectrum (i.e. one without minima due to
        % sidelobes)
        smoothed = conv(input.^2,ones(filterLen,1));
        smoothed = smoothed(floor(filterLen*0.5):end-(filterLen*0.5));
        % now find global minima
        temp = diff(smoothed)';
        % apply mask
        temp(find(temp>0)) = 1;
        temp(find(temp<0)) = -1;
        % find differential of mask to get stationary points
        temp = diff(temp);
        lower = [1,find(temp==2)]+1; % -2 will be maxima, +2 will be minima
        % ACTUALLY THIS NEEDS IMPROVING AS SOMETIMES INDEX IS OUT BY 1
        % (DEPENDS ON WHETHER INDEX IS EVEN OR ODD I THINK)
        % CURRENTLY FIXED BY DOUBLE CHECKING AT THE END OF THIS FUNCTION
        upper = [find(temp==2)+1,N]-1;
        maxima = [find(temp==-2)]+1;
        if(isempty(maxima))
            % just put maxima in middle
            maxima(1) = round(N/2);
        end
        if(maxima(1)>upper(1))
         % ensure that a maximum is located in the first region
            [m,i] = max(input(lower(1):upper(1)));
            maxima = [i,maxima];
        end
        if(maxima(end)<lower(end))
            % ensure that a maximum is located in the last region
            [m,i] = max(input(lower(end):upper(end)));
            maxima(end+1) = i;
        end  
    end        
else
    % no need to find local maxima, we can just use the input
    temp = diff(input); 
    % apply mask
    temp(find(temp>0)) = 1;
    temp(find(temp<0)) = -1;
    % find differential of mask to get stationary points
    temp = diff(temp);
    
    lower = [1;find(temp==2)]; % -2 will be maxima, +2 will be minima
    upper = [find(temp==2)+1;N]; 
end

% MAXIMA CHECK (SEE ABOVE)
for n = 1:length(maxima)
    [dummy,temp] = max(input(lower(n):upper(n)));
    temp = temp+lower(n)-1;
    maxima(n) = temp;
end
%maxima = maxima+lower-1;



        